package com.netifera.platform.net.http.internal.tools;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

import com.netifera.platform.api.model.IModelService;
import com.netifera.platform.api.probe.IProbeManagerService;
import com.netifera.platform.net.dns.model.IDomainEntityFactoryService;
import com.netifera.platform.net.dns.service.nameresolver.INameResolver;
import com.netifera.platform.net.http.web.applications.IWebApplicationDetector;
import com.netifera.platform.net.http.web.model.IWebEntityFactoryService;
import com.netifera.platform.net.model.INetworkEntityFactoryService;
import com.netifera.platform.net.sockets.ISocketEngineService;

public class Activator implements BundleActivator {

	private ServiceTracker modelTracker;
	private ServiceTracker probeManagerTracker;
	private ServiceTracker socketEngineTracker;
	private ServiceTracker networkEntityFactoryTracker;
	private ServiceTracker domainEntityFactoryTracker;
	private ServiceTracker webEntityFactoryTracker;
	private ServiceTracker webApplicationDetectorTracker;
	private ServiceTracker nameResolverTracker;

	private static Activator instance;
	
	public static Activator getInstance() {
		return instance;
	}
	
	public void start(BundleContext context) throws Exception {
		instance = this;
		
		modelTracker = new ServiceTracker(context, IModelService.class.getName(), null);
		modelTracker.open();
		
		probeManagerTracker = new ServiceTracker(context, IProbeManagerService.class.getName(), null);
		probeManagerTracker.open();
		
		socketEngineTracker = new ServiceTracker(context, ISocketEngineService.class.getName(), null);
		socketEngineTracker.open();
		
		networkEntityFactoryTracker = new ServiceTracker(context, INetworkEntityFactoryService.class.getName(), null);
		networkEntityFactoryTracker.open();

		domainEntityFactoryTracker = new ServiceTracker(context, IDomainEntityFactoryService.class.getName(), null);
		domainEntityFactoryTracker.open();

		webEntityFactoryTracker = new ServiceTracker(context, IWebEntityFactoryService.class.getName(), null);
		webEntityFactoryTracker.open();

		webApplicationDetectorTracker = new ServiceTracker(context, IWebApplicationDetector.class.getName(), null);
		webApplicationDetectorTracker.open();
		
		nameResolverTracker = new ServiceTracker(context, INameResolver.class.getName(), null);
		nameResolverTracker.open();
	}

	public void stop(BundleContext context) throws Exception {
		// TODO Auto-generated method stub
		
	}

	public IModelService getModelService() {
		return (IModelService) modelTracker.getService();
	}
	
	public IProbeManagerService getProbeManager() {
		return (IProbeManagerService) probeManagerTracker.getService();
	}
	
	public ISocketEngineService getSocketEngine() {
		return (ISocketEngineService) socketEngineTracker.getService();
	}
	
	public INetworkEntityFactoryService getNetworkEntityFactory() {
		return (INetworkEntityFactoryService) networkEntityFactoryTracker.getService();
	}

	public IDomainEntityFactoryService getDomainEntityFactory() {
		return (IDomainEntityFactoryService) domainEntityFactoryTracker.getService();
	}

	public IWebEntityFactoryService getWebEntityFactory() {
		return (IWebEntityFactoryService) webEntityFactoryTracker.getService();
	}

	public IWebApplicationDetector getWebApplicationDetector() {
		return (IWebApplicationDetector) webApplicationDetectorTracker.getService();
	}
	
	public INameResolver getNameResolver() {
		return (INameResolver) nameResolverTracker.getService();
	}
}
