package com.netifera.platform.net.internal.ui;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.eclipse.swt.graphics.Image;

import com.netifera.platform.api.model.AbstractEntity;
import com.netifera.platform.api.model.IShadowEntity;
import com.netifera.platform.api.model.IStructureContext;
import com.netifera.platform.model.FolderEntity;
import com.netifera.platform.model.TreeStructureContext;
import com.netifera.platform.net.model.ClientEntity;
import com.netifera.platform.net.model.ClientServiceConnectionEntity;
import com.netifera.platform.net.model.CredentialEntity;
import com.netifera.platform.net.model.HostEntity;
import com.netifera.platform.net.model.InternetAddressEntity;
import com.netifera.platform.net.model.NetblockEntity;
import com.netifera.platform.net.model.NetworkAddressEntity;
import com.netifera.platform.net.model.PasswordEntity;
import com.netifera.platform.net.model.PortSetEntity;
import com.netifera.platform.net.model.ServiceEntity;
import com.netifera.platform.net.model.UserEntity;
import com.netifera.platform.net.model.UsernameAndPasswordEntity;
import com.netifera.platform.ui.api.model.IEntityLabelProvider;
import com.netifera.platform.ui.images.ImageCache;
import com.netifera.platform.util.addresses.inet.InternetAddress;
import com.netifera.platform.util.addresses.inet.InternetNetblock;
import com.netifera.platform.util.patternmatching.InternetAddressMatcher;
import com.netifera.platform.util.patternmatching.NetblockMatcher;

public class EntityLabelProvider implements IEntityLabelProvider {
	private final static String PLUGIN_ID = "com.netifera.platform.net.ui";

	private ImageCache images = new ImageCache(PLUGIN_ID);
	
	private final static String HOST_WORKSTATION = "icons/host_workstation.png";
	private final static String HOST_SERVER = "icons/host_server.png";
//	private final static String HOST_MAINFRAME = "icons/host_mainframe.png";
	private final static String HOST_ROUTER = "icons/host_router.png";
//	private final static String HOST_SWITCH = "icons/host_switch.png";
	private final static String HOST_AP = "icons/host_ap.png";
	private final static String HOST_IPHONE = "icons/host_iphone.png";
	
	private final static String NETWORK = "icons/network.png";
	private final static String LOCAL_NETWORK = "icons/network_local.png";
	private final static String PORTS = "icons/ports.png";
	private final static String CREDENTIAL = "icons/credential.png";
	private final static String USER = "icons/user.png";
	private final static String FOLDER = "icons/folder.png";
	private final static String TARGET = "icons/target.png";
	//private final static String HOST_SERVER_IBM = "icons/host_server_ibm.png";
	//private final static String HOST_SERVER_HP = "icons/host_server_hp.png";
	//private final static String HOST_SERVER_AS400 = "icons/host_server_as400.png";
	
	private final static String CISCO_OVERLAY = "icons/cisco_overlay.png";
	private final static String JUNIPER_OVERLAY = "icons/juniper_overlay.png";
	
	private final static String HOST_PRINTER = "icons/host_printer.png";
	
	private final static String SERVICE = "icons/service.png";
	private final static String CLIENT = "icons/client.png";
	private final static String SERVICE_SHELL = "icons/service_shell.png";
	private final static String SERVICE_MAIL = "icons/service_mail.png";
	private final static String SERVICE_FILES = "icons/service_file.png";
	private final static String SERVICE_HTTP = "icons/service_http.png";
	private final static String SERVICE_DATABASE = "icons/service_database.png";
	
	private final static String OS_BSD = "icons/os_bsd_overlay.png";
	//private final static String OS_OPENBSD = "icons/os_openbsd_overlay.png";
	private final static String OS_LINUX = "icons/os_linux_overlay.png";
	private final static String OS_MAC = "icons/os_apple_overlay.png";
	private final static String OS_SUN = "icons/os_sun_overlay.png";
	private final static String OS_WINDOWS = "icons/os_windows_overlay.png";
	//private final static String OS_NOVELL = "icons/os_novell_overlay.png";
	private final static String OS_RTOS = "icons/os_rtos_overlay.png";
	private final static String OS_HPUX = "icons/os_hpux_overlay.png";
	
	
	public String getInformation(IShadowEntity e) {
		if(e instanceof HostEntity) {
			return getHostInformation((HostEntity)e);
		} else if (e instanceof ServiceEntity) {
			return getServiceInformation((ServiceEntity)e);
		} else if (e instanceof ClientEntity) {
			return getClientInformation((ClientEntity)e);
		} else if (e instanceof ClientServiceConnectionEntity) {
			return getServiceInformation(((ClientServiceConnectionEntity)e).getService());
		} else if (e instanceof PortSetEntity) {
			return getPortsetInformation((PortSetEntity)e);
		}
		return null;
	}

	private String getHostInformation(HostEntity e) {
		StringBuffer buffer = new StringBuffer();
		// if has more than 1 address or a label set that prevents from seeing the address
		if (e.getAddresses().size() > 1 || e.getLabel() != null) {
			buffer.append("<p>Addresses: ");
			Iterator<NetworkAddressEntity> addresses = e.getAddresses().iterator();
			while (addresses.hasNext()) {
				NetworkAddressEntity a = addresses.next();
				buffer.append(escape(a.getAddressString()));
				if (addresses.hasNext())
					buffer.append(", ");
			}
			buffer.append("</p>");
		}
		if (e.getNamedAttribute("os") != null) {
			buffer.append("<p>System: "+escape(e.getNamedAttribute("os")));
			if (e.getNamedAttribute("distribution")!=null)
				buffer.append(" - "+escape(e.getNamedAttribute("distribution")));
			if (e.getNamedAttribute("arch") != null) {
				buffer.append(" ("+escape(e.getNamedAttribute("arch"))+")");
			}
			buffer.append("</p>");
		}
		return buffer.toString();
	}

	private String getServiceInformation(ServiceEntity e) {
		if (e.getBanner() == null)
			return null;
		if (e.getServiceType().equals("HTTP"))
			return getHTTPBannerInformation(e.getBanner());
		return "<p>"+escape(truncate(e.getBanner()))+"</p>";
	}

	private String getClientInformation(ClientEntity e) {
		if (e.getBanner() == null)
			return null;
		if (e.getServiceType().equals("HTTP"))
			return getHTTPBannerInformation(e.getBanner());
		return "<p>"+escape(truncate(e.getBanner()))+"</p>";
	}

	private String getHTTPBannerInformation(String data) {
		data = "<p>"+escape(data)+"</p>";
		data = data.replaceAll("<p>Content[^<]+</p>", "");
		data = data.replaceAll("<p>Connection[^<]+</p>", "");
		return data.replaceAll("<p>([^:<]+:)([^<]+)</p>", "<p><span color=\"blue\">$1</span>$2</p>");
	}

	private String truncate(String data) {
		if (data.length() > 400) {
			data = data.substring(0, 400)+"...";
		}
		return data;
	}

	private String escape(String data) {
		data = data.replaceAll("&", "&amp;");
		data = data.replaceAll("<", "&lt;");
		data = data.replaceAll(">", "&gt;");
		data = data.trim().replaceAll("[\\r\\n]+", "</p><p>");
		return data.replaceAll("[^\\p{Print}\\p{Blank}]", "."); // non-printable chars
	}
	
	public String getText(IShadowEntity e) {
		if(e instanceof NetworkAddressEntity) {
			return getAddressText((NetworkAddressEntity)e);
		} else if(e instanceof NetblockEntity) {
			return getNetblockText((NetblockEntity)e);
		} else if(e instanceof HostEntity) {
			return getHostText((HostEntity) e);
		} else if(e instanceof UserEntity) {
			return ((UserEntity)e).getName();
//		} else if(e instanceof LocalNetworkEntity) {
//			return ((LocalNetworkEntity)e).getName();
		} else if(e instanceof FolderEntity) {
			return ((FolderEntity) e).getLabel()+" ("+((TreeStructureContext)((FolderEntity) e).getStructureContext()).getChildren().size()+")";
		} else if(e instanceof PortSetEntity) {
			return getPortsetText((PortSetEntity)e);
		} else if(e instanceof ServiceEntity) {
			return getServiceText((ServiceEntity)e);
		} else if(e instanceof ClientEntity) {
			return getClientText((ClientEntity)e);
		} else if(e instanceof ClientServiceConnectionEntity) {
			return getServiceConnectionText((ClientServiceConnectionEntity)e);
		} else if(e instanceof PasswordEntity) {
			return ((PasswordEntity)e).getPassword();
		} else if(e instanceof UsernameAndPasswordEntity) {
			UsernameAndPasswordEntity credential = (UsernameAndPasswordEntity)e;
			String pw = credential.getPassword();
			if (pw == null || pw.length() == 0) {
				pw = "<no password>";
			}
			return credential.getUsername() + ":" + pw;
		}
		return null;
	}
	
	public String getFullText(IShadowEntity e) {
		if(e instanceof ServiceEntity) {
			ServiceEntity serviceEntity = (ServiceEntity)e;
			return getAddressText(serviceEntity.getAddress())+":"+getText(serviceEntity);
		} else if(e instanceof ClientEntity) {
			ClientEntity clientEntity = (ClientEntity)e;
			return getText(clientEntity.getHost())+" "+getText(clientEntity);
		}
		return getText(e);
	}

	public String getToolTipText(IShadowEntity e) {
		//FIXME this interferes with the action hover
		return null;
	}
	
	public Image getImage(IShadowEntity e) {
		if(e instanceof NetworkAddressEntity) {
			return images.get(NETWORK);
		} else if(e instanceof HostEntity) {
			return getHostImage((HostEntity)e);
		} else if(e instanceof UserEntity) {
			return images.get(USER);
		} else if(e instanceof NetblockEntity) {
			InternetAddress net = ((NetblockEntity)e).getNetblock().getNetworkAddress();
			if (net.isLinkLocal() || net.isPrivate() || net.isLoopback()) { // TODO use InternetNetblock
				return images.get(LOCAL_NETWORK);
			}
			return images.get(NETWORK);
/*		} else if(e instanceof LocalNetworkEntity) {
			InternetAddress net = InternetAddress.fromBytes(((LocalNetworkEntity)e).getAddressData());
			if (net.isLinkLocal() || net.isPrivate() || net.isLoopback()) {
				return localNetworkImage;
			}
			return networkImage;
*/		} else if(e instanceof FolderEntity) {
			return getFolderImage((FolderEntity)e);
		} else if(e instanceof PortSetEntity) {
			return images.get(PORTS);
		} else if(e instanceof ServiceEntity) {
			return getServiceImage((ServiceEntity)e);
		} else if(e instanceof ClientServiceConnectionEntity) {
			return getImage(((ClientServiceConnectionEntity)e).getService());
		} else if(e instanceof ClientEntity) {
			return getOSDecoratedImage(CLIENT, (ClientEntity)e);
		} else if(e instanceof CredentialEntity) {
			return images.get(CREDENTIAL);
		}
		return null;
	}

	private Image getFolderImage(FolderEntity e) {
		if (e.getTag().equals("Target"))
			return images.get(TARGET);
		return images.get(FOLDER);
	}
	
	private Image getHostImage(HostEntity e) {
		String os = e.getNamedAttribute("os");
		if (os == null) os = "";
		os = os.toLowerCase(Locale.ENGLISH);
		String base = HOST_WORKSTATION;
		if (os.matches(".*cisco.*"))
			base = HOST_ROUTER;
		else if (os.matches(".*junos.*"))
			base = HOST_ROUTER;
//		else if (os.matches(".*(as\\/?400|z\\/(os|vm)|os\\/?390).*"))
//			base = HOST_MAINFRAME;
		else if (os.matches(".*printer.*"))
			base = HOST_PRINTER;
		else if (os.matches(".*i(phone|touch).*"))
			base = HOST_IPHONE;
//		else if(e.hasChildren(ServiceEntity.class))
//			base = HOST_SERVER;
		return getOSDecoratedImage(base, e);
	}

	private Image getOSDecoratedImage(String base, AbstractEntity e) {
		String os = e.getNamedAttribute("os");
		if (os == null) os = "";
		os = os.toLowerCase(Locale.ENGLISH);
		if (os.matches(".*linux.*"))
			return images.getDecoratedImage(base, OS_LINUX);
//		if (os.matches(".*openbsd.*"))
//			return images.getDecoratedImage(base, OS_OPENBSD);
		if (os.matches(".*bsd.*"))
			return images.getDecoratedImage(base, OS_BSD);
		if (os.matches(".*(sunos|solaris).*"))
			return images.getDecoratedImage(base, OS_SUN);
		if (os.matches(".*(macos|darwin|osx|os x).*"))
			return images.getDecoratedImage(base, OS_MAC);
		if (os.matches(".*win.*"))
			return images.getDecoratedImage(base, OS_WINDOWS);
//		if (os.matches(".*(novell|netware).*"))
//			return images.getDecoratedImage(base, OS_NOVELL);
		if (os.matches(".*(hp.?ux|(open)?vms).*"))
				return images.getDecoratedImage(base, OS_HPUX);
		if (os.matches(".*vxworks.*"))
			return images.getDecoratedImage(base, OS_RTOS);
		if (os.matches(".*cisco.*"))
			return images.getDecoratedImage(base, CISCO_OVERLAY);
		if (os.matches(".*junos.*"))
			return images.getDecoratedImage(base, JUNIPER_OVERLAY);
//		if (os.matches(".*(as\\/?400|z\\/(os|vm)|os\\/?390).*"))
//			return images.getDecoratedImage(base, IBM_OVERLAY);
		return images.get(base);
	}

	private Image getServiceImage(ServiceEntity e) {
		String type = e.getServiceType();
		if (type.matches(".*SQL.*") || type.equals("Oracle")) {
			return getOSDecoratedImage(SERVICE_DATABASE, e);
		} else if (type.matches("SMTP|POP3|IMAP")) {
			return getOSDecoratedImage(SERVICE_MAIL, e);
		} else if (type.matches("SSH|Telnet")) {
			return getOSDecoratedImage(SERVICE_SHELL, e);
		} else if (type.matches("HTTP|HTTPS")) {
			return getOSDecoratedImage(SERVICE_HTTP, e);
		} else if (type.matches("FTP")) {
			return getOSDecoratedImage(SERVICE_FILES, e);
		}
		return getOSDecoratedImage(SERVICE, e);

	}
	private String getNetblockText(NetblockEntity netblock) {
		return netblock.getNetblock().toString();
	}
	
	private String getHostText(HostEntity host) {
		IStructureContext context;
		try {
			context = host.getStructureContext();
		} catch (IllegalStateException e) {
			context = null;
		}
		
		String tag = "";
		if (context instanceof TreeStructureContext) {
			IShadowEntity parent = ((TreeStructureContext) context).getParent();
			if (parent instanceof FolderEntity)
				tag = "."+((FolderEntity) parent).getTag();
		}
		
		String firstName = null;
		for (NetworkAddressEntity address: host.getAddresses()) {
			if (address instanceof InternetAddressEntity) {
				for (String name: ((InternetAddressEntity)address).getNames()) {
					if (firstName == null) {
						firstName = name;
					}
					if (name.endsWith(tag)) {
						firstName = name;
						break;
					}
				}
			}
		}

		if (firstName != null) {
			StringBuffer buf = new StringBuffer();
			buf.append(firstName);

			List<String> otherNames = new ArrayList<String>();
			for (NetworkAddressEntity address: host.getAddresses()) {
				if (address instanceof InternetAddressEntity) {
					for (String name: ((InternetAddressEntity)address).getNames()) {
						if (!name.equals(firstName))
							otherNames.add(name);
					}
				}
			}

			if (otherNames.size()>0) {
				buf.append(" [");
				int last = otherNames.size()-1;
				for (int i=0; i<otherNames.size(); i++) {
					buf.append(otherNames.get(i));
					if (i != last) {
						if (buf.length()>40) {
							buf.append("..");
							break;
						}
						buf.append(", ");
					}
				}
				buf.append("]");
			}
			
			List<NetworkAddressEntity> addresses = host.getAddresses();
			buf.append(" (");
			int last = addresses.size()-1;
			for (int i=0; i<addresses.size(); i++) {
				buf.append(addresses.get(i).getAddress());
				if (i != last) {
					if (buf.length()>50) {
						buf.append("..");
						break;
					}
					buf.append(", ");
				}
			}
			buf.append(")");
			
			return buf.toString();
		}
		
		return host.getLabel() == null ? host.getDefaultAddress().getAddressString() : host.getLabel();
	}
	
	private String getAddressText(NetworkAddressEntity e) {
		StringBuffer buffer = new StringBuffer();
		buffer.append(e.getAddress());
		if (e instanceof InternetAddressEntity) {
			InternetAddressEntity i = (InternetAddressEntity) e;
			if (i.getNames().size() > 0) {
				buffer.append(" (");
				Iterator<String> iterator = i.getNames().iterator();

				while (iterator.hasNext()) {
					buffer.append(iterator.next());
					if (iterator.hasNext())
						buffer.append(", ");
				}
				buffer.append(")");
			}
		}
		return buffer.toString();
	}

	private String getClientText(ClientEntity e) {
		StringBuilder label = new StringBuilder();
		
		label.append(e.getServiceType()+" client");
						
		if (e.getProduct() != null && e.getProduct().length() > 0) {
			label.append(" [" + e.getProduct());
			
			if (e.getVersion() != null && e.getVersion().length() > 0) {
				label.append(" " + e.getVersion());
			}
			label.append("]");
		}
		
		return label.toString();
	}
	
	private String getServiceText(ServiceEntity e) {
		StringBuilder label = new StringBuilder();
				
//		if(useServiceAddress) {
//			InternetAddressEntity address = new InternetAddressEntity(e.getParent());
//			label.append(address.getKey() + ":");
//		}
		label.append(e.getPort()+"/"+e.getProtocol()+" ");
		
		label.append(e.getServiceType()+" ");

		if (e.getProduct() != null && e.getProduct().length() > 0 && !e.getProduct().equals(e.getServiceType())) {
			label.append(" ["+e.getProduct());
			if (e.getVersion() != null && e.getVersion().length() > 0)
				label.append(" "+e.getVersion());
			label.append("]");
		} else if (e.getVersion() != null && e.getVersion().length()>0) {
			label.append(" ["+e.getVersion()+"]");
		}
		
		return label.toString();
	}

	private String getServiceConnectionText(ClientServiceConnectionEntity e) {
		InternetAddressEntity address = e.getService().getAddress();
		HostEntity host = address.getHost();
		
		StringBuffer buffer = new StringBuffer();
		
		if (host.getLabel() != null) {
			buffer.append(host.getLabel());
		} else {
			buffer.append(address.getAddress().toStringLiteral());
		}
		buffer.append(':'); // FIXME '/' for IPv6?
		buffer.append(getServiceText(e.getService()));
		if (e.getIdentity() != null) {
			buffer.append('(' + e.getIdentity() + ')');
		}
		return buffer.toString();
	}
	
	private final static int PORTS_FORMATTED_LINE_LENGTH = 40;
	private final static int PORTS_TRUNCATE_LENGTH = 50;

	private String getPortsetText(PortSetEntity portset) {
		return portset.getProtocol().toUpperCase(Locale.ENGLISH) + " [" + getPortString(portset) + "]";
		
	}
	
	private String getPortString(PortSetEntity portset) {
		final String ports = portset.getPorts();
		if(ports.length() < PORTS_TRUNCATE_LENGTH) return ports;
		int idx = ports.lastIndexOf(',', PORTS_TRUNCATE_LENGTH);
		if(idx == -1) return ports;
		return ports.substring(0,idx) + "...";
		
	}
	
	private String getPortsetInformation(PortSetEntity portset) {
		final List<String> lines = breakIntoLines(portset.getPorts());

		final StringBuilder result = new StringBuilder();
		result.append("<P>Listening " + portset.getProtocol().toUpperCase(Locale.ENGLISH) + " Ports:</P>");
		for(String line: lines) {
			result.append("<P>");
			result.append(line.replaceAll(",", ", "));
			result.append("</P>\n");
		}
		return result.toString();
	}
	
	private List<String> breakIntoLines(String ports) {
		String remaining = ports;
		List<String> lines = new ArrayList<String>();
		while(remaining.length() > PORTS_FORMATTED_LINE_LENGTH) {
			int idx = remaining.lastIndexOf(',', PORTS_FORMATTED_LINE_LENGTH);
			if(idx == -1) {
				// Should not happen
				lines.add(remaining);
				return lines;
			}
			lines.add(remaining.substring(0, idx + 1));
			remaining = remaining.substring(idx + 1);
		}
		if(remaining.length() > 0) 
			lines.add(remaining);
		return lines;
	}
	
	public void dispose() {
		images.dispose();
	}


	public int getSortingCategory(IShadowEntity e) {
		
		/* top level ordering */
		if(e instanceof FolderEntity) {
			FolderEntity folder = (FolderEntity) e;
			if(folder.getTag().equals("Target"))
				return 0;
			else if(NetblockMatcher.matchesIPv4(folder.getTag()))
				return 5;
			else if(NetblockMatcher.matchesIPv6(folder.getTag()))
				return 6;
			else
				return 3;
		}
		
		/* ordering below host entity */
		if(e instanceof PortSetEntity) {
			return 0;
		}
		if(e instanceof ServiceEntity) {
			return 2;
		}
		/* ordering inside network folders */
		if(e instanceof NetblockEntity) {
			return 0;
		}
		if(e instanceof HostEntity) {
			return 1;
		}
		return -1;
	}

	public int compare(IShadowEntity e1, IShadowEntity e2) {
		if(e1 instanceof ServiceEntity && e2 instanceof ServiceEntity) {
			return compareServiceEntities((ServiceEntity)e1, (ServiceEntity) e2);
		}
		if(e1 instanceof HostEntity && e2 instanceof HostEntity) {
			return compareHostEntities((HostEntity)e1, (HostEntity) e2);
		}
		if(e1 instanceof FolderEntity && e2 instanceof FolderEntity) {
			return compareFolderEntities((FolderEntity)e1, (FolderEntity) e2);
		}
		return -2;
	}
	
	private int compareServiceEntities(ServiceEntity e1, ServiceEntity e2) {
		
		if(e1.getPort() < e2.getPort()) 
			return -1;
		else if(e1.getPort() > e2.getPort())
			return 1;
		else 
			return e1.getProtocol().compareToIgnoreCase(e2.getProtocol());
	}
	
	// TODO review (james: compare addresses?)
	private int compareHostEntities(HostEntity e1, HostEntity e2) {
		String s1 = getHostText(e1).split("\\s")[0];
		String s2 = getHostText(e2).split("\\s")[0];
		InternetAddressMatcher m1 = new InternetAddressMatcher(s1);
		InternetAddressMatcher m2 = new InternetAddressMatcher(s2);
		
		if (m1.matches() && m2.matches()) {
			return InternetAddress.fromString(s1).compareTo(InternetAddress.fromString(s2));
		}
		
		return -2;
	}
	
	private int compareFolderEntities(FolderEntity e1, FolderEntity e2) {
		String tag1 = e1.getTag();
		String tag2 = e2.getTag();
		
		// need to do v4 and v6 separately
		if (NetblockMatcher.matchesIPv4(tag1) && NetblockMatcher.matchesIPv4(tag2)) {
			return InternetNetblock.fromString(tag1).compareTo(
					InternetNetblock.fromString(tag2));
		}
		else if (NetblockMatcher.matchesIPv6(tag1) && NetblockMatcher.matchesIPv6(tag2)) {
			return InternetNetblock.fromString(tag1).compareTo(
					InternetNetblock.fromString(tag2));
		}

		return -2;
	}
}
