package com.netifera.platform.ui.images;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.DecorationOverlayIcon;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;


public class ImageCache {

	private final Map<ImageDescriptor, Image> imageMap = new HashMap<ImageDescriptor, Image>();
	private final String pluginId;

	
	public ImageCache(String id) {
		pluginId = id;
	}

	public Image get(String key) {
		return lookupImage(getDescriptor(key));
	}

	public ImageDescriptor getDescriptor(String key) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(pluginId, key);
	}

	private Image lookupImage(ImageDescriptor descriptor) {

		if (descriptor == null) {
			return null;
		}

		synchronized (imageMap) {
			/* if descriptor not contained in map create image and put it */
			if (!imageMap.containsKey(descriptor)) {
				imageMap.put(descriptor, descriptor.createImage());
			}

			return imageMap.get(descriptor);
		}
	}

	public Image getDecoratedImage(String key, String overlayKey) {
		Image baseImage = get(key);
		if (baseImage == null) {
			return null;
		}

		ImageDescriptor overlayDescriptor = getDescriptor(overlayKey);
		if (overlayDescriptor == null) {
			return null;
		}

		ImageDescriptor descriptor = new DecorationOverlayIcon(baseImage,
				overlayDescriptor, IDecoration.BOTTOM_LEFT);

		return lookupImage(descriptor);
	}

	public synchronized void dispose() {
		for (Image image : imageMap.values()) {
			image.dispose();
		}
		imageMap.clear();
	}
}
