package com.netifera.platform.ui.internal.spaces.visualizations;

import org.eclipse.jface.dialogs.PopupDialog;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;

import com.netifera.platform.api.model.ISpace;
import com.netifera.platform.ui.spaces.actions.ActionHover;
import com.netifera.platform.ui.spaces.tree.SpaceTreeContentProvider;
import com.netifera.platform.ui.spaces.tree.SpaceTreeLabelProvider;
import com.netifera.platform.ui.spaces.tree.TreeViewerComparator;
import com.netifera.platform.ui.spaces.visualizations.ISpaceVisualizationProvider;
import com.netifera.platform.ui.util.MouseTracker;

public class SpaceTreeVisualizationProvider implements ISpaceVisualizationProvider {

	
	public String getName() {
		return "Tree";
	}

	public ContentViewer create(Composite parent, ISpace space) {
		final TreeViewer treeViewer = new TreeViewer(parent);
		// Work around for bug #19
		if(isOSX()) {
			treeViewer.getTree().setHeaderVisible(true);
		}
		treeViewer.setContentProvider(new SpaceTreeContentProvider());
		treeViewer.setLabelProvider(new SpaceTreeLabelProvider());
		treeViewer.setInput(space);
		treeViewer.setComparator(new TreeViewerComparator());

		/* implement the mouse tracker the action hover handlers*/
		final MouseTracker mouseTracker = new MouseTracker(treeViewer.getTree()) {
			private PopupDialog informationControl;

			@Override
			protected Object getItemAt(Point point) {
				TreeItem treeItem = treeViewer.getTree().getItem(point);
				if (treeItem != null) {
					return treeItem.getData();
				}
				return null;
			}

			@Override
			protected Rectangle getAreaOfItemAt(Point point) {
				TreeItem treeItem = treeViewer.getTree().getItem(point);
				if (treeItem != null) {
					Rectangle treeItemArea = treeItem.getBounds();

					/*
					 * the TreeItem getBounds rectangle only includes the text. But
					 * getItem(point) returns the rectangle for any point in the
					 * tree row.
					 */
					return expandedItemArea(treeItemArea);
				}
				return super.getAreaOfItemAt(point);
			}
			
			@Override
			protected Rectangle getAreaOfSelectedItem() {
				TreeItem[] selection = treeViewer.getTree().getSelection();
				if(selection != null && selection.length > 0) {
					return selection[0].getBounds();
				}
				return null;
			}
			
			private Rectangle expandedItemArea(Rectangle itemArea) {
				return new Rectangle(Math.max(itemArea.x - 12, 2), Math.max(itemArea.y
						- EPSILON * 2, 0), itemArea.width + 12 * 2, itemArea.height
						+ EPSILON * 2 * 2);
			}
			
			@Override
			protected void showInformationControl(Shell parent, Point location,
					Object input, Object item) {
				informationControl = new ActionHover(parent, location, input, item);
				informationControl.open();
			}
			@Override
			protected void hideInformationControl() {
				if(informationControl != null) {
					informationControl.close();
				}
			}
			@Override
			protected boolean focusInformationControl() {
				if(informationControl != null) {
					Shell shell = informationControl.getShell();
					if(shell != null) {
						return shell.setFocus();
					}
				}
				return false;
			}
			@Override
			protected Rectangle getInformationControlArea() {
				if(informationControl != null) {
					Shell shell = informationControl.getShell();
					if(shell != null) {
						return shell.getBounds();
					}
				}
				return null;
			}
		};
		
		treeViewer.getControl().addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent event) {
				mouseTracker.stop();
			}
		});
		
		return treeViewer;
	}
	
	private boolean isOSX() {
		final String os =  System.getProperty("osgi.os");
		return (os != null && os.equals("macosx"));
	}
}
