package com.netifera.platform.ui.model.internal;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import com.netifera.platform.api.model.IShadowEntity;
import com.netifera.platform.ui.UIPlugin;
import com.netifera.platform.ui.api.model.IEntityLabelProvider;
import com.netifera.platform.ui.api.model.IEntityLabelProviderService;
import com.netifera.platform.ui.images.ImageCache;

public class EntityLabelProviderService implements IEntityLabelProviderService {
		
	private final static String UNKNOWN_IMAGE = "icons/unknown_entity.png";
	private final ImageCache images = new ImageCache(UIPlugin.PLUGIN_ID);
	
	private final List<IEntityLabelProvider> labelProviders = new ArrayList<IEntityLabelProvider>();

	public synchronized String getToolTipText(IShadowEntity entity) {
		for(IEntityLabelProvider provider : labelProviders) {
			String toolTip = provider.getToolTipText(entity);
			if(toolTip != null)
				return toolTip;
		}
		// not found
		return null;
	}

	public synchronized String getText(IShadowEntity entity) {
		for(IEntityLabelProvider provider : labelProviders) {
			String text = provider.getText(entity);
			if(text != null)
				return text;
		}
		// not found
		return (entity.getTypeName().matches("[AEIOU].*") ? "an " : "a ") + entity.getTypeName();
	}

	public synchronized String getFullText(IShadowEntity entity) {
		for(IEntityLabelProvider provider : labelProviders) {
			String text = provider.getFullText(entity);
			if(text != null)
				return text;
		}
		// not found
		return getText(entity);
	}

	public synchronized Image getImage(IShadowEntity entity) {
		for(IEntityLabelProvider provider : labelProviders) {
			Image image = provider.getImage(entity);
			if(image != null)
				return image;
		}
		// not found
		return images.get(UNKNOWN_IMAGE);
	}
	
	public synchronized String getInformation(IShadowEntity entity) {
		StringBuffer buffer = new StringBuffer();
		for(IEntityLabelProvider provider : labelProviders) {
			String information = provider.getInformation(entity);
			if(information != null)
				buffer.append(information);
		}
		return buffer.toString();
	}
	
	public synchronized int getSortingCategory(IShadowEntity entity) {
		for(IEntityLabelProvider provider : labelProviders) {
			int category = provider.getSortingCategory(entity);
			if(category != -1)
				return category;
		}
		return 10;
	}

	
	protected synchronized void registerLabelProvider(IEntityLabelProvider provider) {
		labelProviders.add(provider);	
	}
	
	protected synchronized void unregisterLabelProvider(IEntityLabelProvider provider) {
		labelProviders.remove(provider);	
	}

	public int compare(IShadowEntity e1, IShadowEntity e2) {
		for(IEntityLabelProvider provider : labelProviders) {
			int r = provider.compare(e1, e2);
			if(r != -2)
				return r;
		}
		return -2;
	}
}
