package com.netifera.platform.ui.spaces.editors;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPageListener;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.part.EditorPart;

import com.netifera.platform.ui.internal.spaces.Activator;
import com.netifera.platform.ui.probe.status.ProbeStatusLine;
import com.netifera.platform.ui.spaces.SpaceEditorInput;

public class SpaceProbeStatusLine extends ProbeStatusLine {

	private IPartListener partListener;
	
	public SpaceProbeStatusLine(String id) {
		super(id);
	}

	public void fill(Composite parent) {
		super.fill(parent);
		initializeListeners();
		editorChanged();
	}
	
	private void initializeListeners() {
		final IWorkbenchWindow window = getWindow();
		if(window == null) {
			return;
		}
		
		partListener = createPartListener();
		window.addPageListener(createPageListener());
		final IWorkbenchPage page = window.getActivePage();
		if(page != null) {
			page.addPartListener(partListener);
		}
	}
	
	private IPartListener createPartListener() {
		return new IPartListener() {

			public void partActivated(IWorkbenchPart part) {
				if(part instanceof EditorPart) 
					editorChanged();
			}

			public void partClosed(IWorkbenchPart part) {
				if(part instanceof EditorPart) 
					editorChanged();			
			}
			
			public void partOpened(IWorkbenchPart part) {
				if(part instanceof EditorPart) 
					editorChanged();
			}
			public void partBroughtToTop(IWorkbenchPart part) {}
			public void partDeactivated(IWorkbenchPart part) {}		
		};	
	}
	
	private IPageListener createPageListener() {
	    /* As pages are opened and closed attach or detach partListener from them */
		return new IPageListener() {
			public void pageActivated(IWorkbenchPage page) {}
			public void pageClosed(IWorkbenchPage page) { page.removePartListener(partListener); }
			public void pageOpened(IWorkbenchPage page) { page.addPartListener(partListener); }
		};
	}
	
	private void editorChanged() {
		IEditorPart editor = getActiveEditor();
		
		if(editor == null ||
				!(editor.getEditorInput() instanceof SpaceEditorInput)) {
			return;
		}
		
		SpaceEditorInput input = (SpaceEditorInput) editor.getEditorInput();
		
		update(input.getProbeForSpace());
	}
	
	private IEditorPart getActiveEditor() {
		final IWorkbenchWindow window = getWindow();
		if(window == null)
			return null;
		
		final IWorkbenchPage activePage = window.getActivePage();
		if(activePage == null) 
			return null;
		return activePage.getActiveEditor();
	}
	
	private IWorkbenchWindow getWindow() {
//		return getSite().getWorkbenchWindow();
		return Activator.getDefault().getWorkbench().getActiveWorkbenchWindow();
	}
}
